﻿/*

	© 2009 360Data
	http://www.360data.nl
	

	This script replaces all the "Polish" characters (characters inserted via CP 1250 machines in Poland)
	with their Unicode equivalents.

	This should only be executed in its present form on a db without triggers or with all DML triggers 
	temporarily disabled and is only suitable for a SQL 2005 or later database.	


	SubChar RealChar SubCode     RealCode
	------- -------- ----------- -----------
	£       Ł        163         321
	³       ł        179         322
	¥       Ą        165         260
	¹       ą        185         261
	Æ       Ć        198         262
	æ       ć        230         263
	Ê       Ę        202         280
	ê       ę        234         281
	Œ       Ś        338         346
	œ       ś        339         347
	       Ź         143         377
	Ÿ       ź        376         378
	¯       Ż        175         379
	¿       ż        191         380
	Ñ       Ń        209         323
	ñ       ń        241         324

*/

use	[Database]
go

declare	@v_SubList	table
	(SubCode	int,
	RealCode	int);

declare	@v_ColList	table
	(TableName	nvarchar(128),
	ColName		nvarchar(128));		

declare	@v_CmdString	nvarchar(max);
declare	@v_TableName	nvarchar(128);
declare	@v_ColName	nvarchar(128);
declare	@v_CurrSubCode	int;
declare	@v_MinSubCode	int;
declare	@v_RealCode	int;

set nocount on;

insert	into @v_SubList	(SubCode, RealCode)	--	Populate the list of characters and replacements
	values(163, 321);

insert	into @v_SubList	(SubCode, RealCode)
	values(179, 322);

insert	into @v_SubList	(SubCode, RealCode)
	values(165, 260);

insert	into @v_SubList	(SubCode, RealCode)
	values(185, 261);

insert	into @v_SubList	(SubCode, RealCode)
	values(198, 262);

insert	into @v_SubList	(SubCode, RealCode)
	values(230, 263);

insert	into @v_SubList	(SubCode, RealCode)
	values(202, 280);

insert	into @v_SubList	(SubCode, RealCode)
	values(234, 281);

insert	into @v_SubList	(SubCode, RealCode)
	values(338, 346);

insert	into @v_SubList	(SubCode, RealCode)
	values(339, 347);

insert	into @v_SubList	(SubCode, RealCode)
	values(143, 377);

insert	into @v_SubList	(SubCode, RealCode)
	values(376, 378);

insert	into @v_SubList	(SubCode, RealCode)
	values(175, 379);

insert	into @v_SubList	(SubCode, RealCode)
	values(191, 380);

insert	into @v_SubList	(SubCode, RealCode)
	values(209, 323);

insert	into @v_SubList	(SubCode, RealCode)
	values(241, 324);

/*
select	nchar(SubCode) as SubChar, SubCode, 
	nchar(RealCode) as RealChar, RealCode
from	@v_SubList
*/


--	Populate the list of tables and columns to be updated

insert into @v_ColList (TableName, ColName)
	select	object_name(c.object_id) as TableName, c.name as ColName
	from	sys.columns	c
	join	sys.objects	o
	on	o.object_id =	c.object_id
	where	c.system_type_id in	(231, 239)	
	and	o.type_desc =	N'USER_TABLE'	
	order	by o.name, c.column_id;
	

begin try

	declare	TableCursor cursor
	for

		-- Cursor will loop through all columns

		select	TableName, ColName
		from	@v_ColList
		where	not ColName like N'%_code'
		order	by TableName, ColName

	open	TableCursor

		fetch next from TableCursor into @v_TableName, @v_ColName

		while	@@fetch_status <> -1
		begin
			
			print	N'Updating ' + rtrim(@v_TableName) + '.' + rtrim(@v_ColName) + '...'

			--	Loop through the list of all possible characters to be replaced
			--	for each table column in the cursor

			select	@v_CurrSubCode =	max(SubCode),
				@v_MinSubCode =		min(SubCode)
			from	@v_SubList

	
			while	@v_CurrSubCode >=	@v_MinSubCode
			begin

				select	@v_RealCode =	RealCode
				from	@v_SubList	
				where	SubCode =	@v_CurrSubCode;


				print	N'Replacing ' + nchar(@v_CurrSubCode) + ' with ' + 
					nchar(@v_RealCode) + '...'

				set	@v_CmdString =	N'update ' + rtrim(@v_TableName) + 
							' set ' + rtrim(@v_ColName) + ' = replace(' +
							rtrim(@v_ColName) + 
							' collate SQL_Latin1_General_CP1_CS_AS, nchar(' + 
							cast(@v_CurrSubCode as nchar(3)) + '), N''' +
							nchar(@v_RealCode) + ''')' 
	
							-- + ' where FILTER CONDITION; ' 

				 -- print	@v_CmdString;

				exec	(@v_CmdString);

				select	@v_CurrSubCode = max(SubCode)
				from	@v_SubList
				where	SubCode <	@v_CurrSubCode;

			end

			fetch next from TableCursor into @v_TableName, @v_ColName

		end;

	close		TableCursor
	deallocate	TableCursor;


end	try
begin	catch

	select	error_message();	

	close		TableCursor;
	deallocate	TableCursor;

end	catch;


